@@ -4,7 +4,7 @@ module HLint.Default where
 import Control.Arrow
 import Control.Exception
 import Control.Monad
-import Control.Monad.State
+import Control.Monad.Trans.State
 import qualified Data.Foldable
 import Data.Foldable(asum, sequenceA_, traverse_, for_)
 import Data.Traversable(traverse, for)
@@ -61,8 +61,8 @@ error = compare x y /= GT ==> x <= y
 error = compare x y == LT ==> x < y
 error = compare x y /= LT ==> x >= y
 error = compare x y == GT ==> x > y
-error = x == a || x == b || x == c ==> x `elem` [a,b,c] where note = ValidInstance "Eq" x
-error = x /= a && x /= b && x /= c ==> x `notElem` [a,b,c] where note = ValidInstance "Eq" x
+--warning = x == a || x == b || x == c ==> x `elem` [a,b,c] where note = ValidInstance "Eq" x
+--warning = x /= a && x /= b && x /= c ==> x `notElem` [a,b,c] where note = ValidInstance "Eq" x
 --error = compare (f x) (f y) ==> Data.Ord.comparing f x y -- not that great
 --error = on compare f ==> Data.Ord.comparing f -- not that great
 error = head (sort x) ==> minimum x
@@ -92,6 +92,7 @@ warn  = x !! 0 ==> head x
 error = take n (repeat x) ==> replicate n x
 error = map f (replicate n x) ==> replicate n (f x)
 error = map f (repeat x) ==> repeat (f x)
+error = cycle [x] ==> repeat x
 error = head (reverse x) ==> last x
 error = head (drop n x) ==> x !! n where _ = isNat n
 error = reverse (tail (reverse x)) ==> init x where note = IncreasesLaziness
@@ -100,6 +101,8 @@ error "Avoid reverse" = reverse (reverse x) ==> x where note = IncreasesLaziness
 error = isPrefixOf (reverse x) (reverse y) ==> isSuffixOf x y
 error = foldr (++) [] ==> concat
 error = foldl (++) [] ==> concat where note = IncreasesLaziness
+error = foldl f (head x) (tail x) ==> foldl1 f x
+error = foldr f (last x) (init x) ==> foldr1 f x
 error = span (not . p) ==> break p
 error = break (not . p) ==> span p
 error = (takeWhile p x, dropWhile p x) ==> span p x
@@ -108,6 +111,7 @@ error = snd (span p x) ==> dropWhile p x
 error = fst (break p x) ==> takeWhile (not . p) x
 error = snd (break p x) ==> dropWhile (not . p) x
 error = concatMap (++ "\n") ==> unlines
+error = intercalate "\n" x ++ "\n" ==> unlines x
 error = map id ==> id
 error = or (map p x) ==> any p x
 error = and (map p x) ==> all p x
@@ -150,7 +154,20 @@ warn "Use null" = length x > 0 ==> not (null x) where note = IncreasesLaziness
 warn "Use null" = length x >= 1 ==> not (null x) where note = IncreasesLaziness
 error "Take on a non-positive" = take i x ==> [] where _ = isNegZero i
 error "Drop on a non-positive" = drop i x ==> x where _ = isNegZero i
+error = last (scanl f z x) ==> foldl f z x
+error = head (scanr f z x) ==> foldr f z x
 
+-- BY
+
+error = deleteBy (==) ==> delete
+error = groupBy (==) ==> group
+error = insertBy compare ==> insert
+error = intersectBy (==) ==> intersect
+error = maximumBy compare ==> maximum
+error = minimumBy compare ==> minimum
+error = nubBy (==) ==> nub
+error = sortBy compare ==> sort
+error = unionBy (==) ==> union
 
 -- FOLDS
 
@@ -187,9 +204,12 @@ warn "Use curry" = (\x y -> f (x,y)) ==> curry f
 warn "Use uncurry" = (\(x,y) -> f x y) ==> uncurry f where note = IncreasesLaziness
 error "Redundant $" = (($) . f) ==> f
 error "Redundant $" = (f $) ==> f
-warn  = (\x -> y) ==> const y where _ = isAtom y
+warn  = (\x -> y) ==> const y where _ = isAtom y && notIn x y
 error "Redundant flip" = flip f x y ==> f y x where _ = isApp original
 warn  = (\a b -> g (f a) (f b)) ==> g `Data.Function.on` f
+error "Evaluate" = id x ==> x
+error "Redundant id" = id . x ==> x
+error "Redundant id" = x . id ==> x
 
 -- CHAR
 
@@ -199,7 +219,9 @@ error = a >= '0' && a <= '9' ==> isDigit a
 error = a >= '0' && a <= '7' ==> isOctDigit a
 error = not (isControl a) ==> isPrint a
 error = isLower a || isUpper a ==> isAlpha a
+error = isUpper a || isLower a ==> isAlpha a
 error = isAlpha a || isDigit a ==> isAlphaNum a
+error = isDigit a || isAlpha a ==> isAlphaNum a
 
 -- BOOL
 
@@ -249,6 +271,8 @@ warn "Redundant pair" = (fst x, snd x) ==>  x where note = DecreasesLaziness
 
 error "Functor law" = fmap f (fmap g x) ==> fmap (f . g) x
 error "Functor law" = fmap id ==> id
+warn = fmap f $ x ==> f Control.Applicative.<$> x
+    where _ = isApp x || isAtom x
 
 -- MONAD
 
@@ -269,14 +293,15 @@ error = when (not x) ==> unless x
 error = x >>= id ==> Control.Monad.join x
 error = liftM f (liftM g x) ==> liftM (f . g) x
 error = fmap f (fmap g x) ==> fmap (f . g) x
-warn = a >> return () ==> void a
-warn = fmap (const ()) ==> void
+warn  = a >> return () ==> Control.Monad.void a
+    where _ = isAtom a || isApp a
+error = fmap (const ()) ==> Control.Monad.void
 error = flip (>=>) ==> (<=<)
 error = flip (<=<) ==> (>=>)
-error = (\x -> f x >>= g) ==> f Control.Monad.>=> g
-error = (\x -> f =<< g x) ==> f Control.Monad.<=< g
+warn  = (\x -> f x >>= g) ==> f Control.Monad.>=> g
+warn  = (\x -> f =<< g x) ==> f Control.Monad.<=< g
 error = a >> forever a ==> forever a
-warn = liftM2 id ==> ap
+warn  = liftM2 id ==> ap
 error = mapM (uncurry f) (zip l m) ==> zipWithM f l m
 
 -- STATE MONAD
@@ -361,13 +386,13 @@ error = [a | Right a <- a] ==> rights a
 
 -- INFIX
 
-warn "Use infix" = X.elem x y ==> x `X.elem` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = X.notElem x y ==> x `X.notElem` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = X.isInfixOf x y ==> x `X.isInfixOf` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = X.isSuffixOf x y ==> x `X.isSuffixOf` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = X.isPrefixOf x y ==> x `X.isPrefixOf` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = X.union x y ==> x `X.union` y where _ = not (isInfixApp original) && not (isParen result)
-warn "Use infix" = X.intersect x y ==> x `X.intersect` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = elem x y ==> x `elem` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = notElem x y ==> x `notElem` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = isInfixOf x y ==> x `isInfixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = isSuffixOf x y ==> x `isSuffixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = isPrefixOf x y ==> x `isPrefixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = union x y ==> x `union` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = intersect x y ==> x `intersect` y where _ = not (isInfixApp original) && not (isParen result)
 
 -- MATHS
 
@@ -375,6 +400,7 @@ error "Redundant fromIntegral" = fromIntegral x ==> x where _ = isLitInt x
 error "Redundant fromInteger" = fromInteger x ==> x where _ = isLitInt x
 warn  = x + negate y ==> x - y
 warn  = 0 - x ==> negate x
+error "Redundant negate" = negate (negate x) ==> x
 warn  = log y / log x ==> logBase x y
 warn  = sin x / cos x ==> tan x
 warn  = sinh x / cosh x ==> tanh x
@@ -400,7 +426,6 @@ warn = Control.Exception.bracket b (const a) (const t) ==> Control.Exception.bra
 warn = Control.Exception.bracket (openFile x y) hClose ==> withFile x y
 warn = Control.Exception.bracket (openBinaryFile x y) hClose ==> withBinaryFile x y
 warn = throw (ErrorCall a) ==> error a
-error = a `seq` return a ==> Control.Exception.evaluate a
 error = toException NonTermination ==> nonTermination
 error = toException NestedAtomically ==> nestedAtomically
 
@@ -452,7 +477,6 @@ error "Evaluate" = x / 1 ==> x
 error "Evaluate" = concat [a] ==> a
 error "Evaluate" = concat [] ==> []
 error "Evaluate" = zip [] [] ==> []
-error "Evaluate" = id x ==> x
 error "Evaluate" = const x y ==> x
 
 -- COMPLEX
@@ -489,7 +513,7 @@ yes = not (a == b) -- a /= b
 yes = not (a /= b) -- a == b
 yes = if a then 1 else if b then 1 else 2 -- if a || b then 1 else 2
 no  = if a then 1 else if b then 3 else 2
-yes = a >>= return . id -- Control.Monad.liftM id a
+yes = a >>= return . bob -- Control.Monad.liftM bob a
 yes = (x !! 0) + (x !! 2) -- head x
 yes = if b < 42 then [a] else [] -- [a | b < 42]
 no  = take n (foo xs) == "hello"
@@ -534,10 +558,8 @@ error = a $$$$ b $$$$ c ==> a . b $$$$$ c
 yes = when (not . null $ asdf) -- unless (null asdf)
 yes = id 1 -- 1
 yes = case concat (map f x) of [] -> [] -- concatMap f x
-yes = Map.union a b -- a `Map.union` b
 yes = [v | v <- xs] -- xs
 no  = [Left x | Left x <- xs]
-yes = Map.union a b -- a `Map.union` b
 when p s = if p then s else return ()
 no = x ^^ 18.5
 instance Arrow (->) where first f = f *** id
@@ -553,7 +575,6 @@ foo = evaluate [12] -- return [12]
 test = \ a -> f a >>= \ b -> return (a, b)
 fooer input = catMaybes . map Just $ input -- mapMaybe Just
 main = print $ map (\_->5) [2,3,5] -- const 5
-main = x == a || x == b || x == c -- x `elem` [a,b,c]
 main = head $ drop n x
 main = head $ drop (-3) x -- x
 main = head $ drop 2 x -- x !! 2
@@ -564,6 +585,26 @@ main = take (-y) x
 main = take 4 x
 main = let (first, rest) = (takeWhile p l, dropWhile p l) in rest -- span p l
 main = map $ \ d -> ([| $d |], [| $d |])
+pairs (x:xs) = map (\y -> (x,y)) xs ++ pairs xs
+{-# ANN foo "HLint: ignore" #-};foo = map f (map g x) -- @Ignore ???
+yes = fmap lines $ abc 123 -- lines Control.Applicative.<$> abc 123
+no = fmap lines $ abc $ def 123
+test = foo . not . not -- id
+test = map (not . not) xs -- id
+used = not . not . any (`notElem` special) . fst . derives -- any (`notElem` special) . fst . derives
+test = foo . id . map -- map
+test = food id xs
+yes = baz baz >> return () -- Control.Monad.void (baz baz)
+no = foo >>= bar >>= something >>= elsee >> return ()
+no = f (#) x
+data Pair = P {a :: !Int}; foo = return $! P{a=undefined}
+data Pair = P {a :: !Int}; foo = return $! P undefined
+foo = return $! Just undefined -- return (Just undefined)
+foo = return $! (a,b) -- return (a,b)
+foo = return $! 1
+foo = return $! "test"
+bar = [x| (x,_) <- pts]
+return' x = x `seq` return x
 
 import Prelude \
 yes = flip mapM -- Control.Monad.forM
