@@ -2,6 +2,7 @@
 
 module HSE.Util where
 
+import Control.Applicative
 import Control.Monad
 import Data.List
 import Data.Maybe
@@ -85,7 +86,6 @@ isMDo MDo{} = True; isMDo _ = False
 isBoxed Boxed{} = True; isBoxed _ = False
 isDerivDecl DerivDecl{} = True; isDerivDecl _ = False
 isPBangPat PBangPat{} = True; isPBangPat _ = False
-isPExplTypeArg PExplTypeArg{} = True; isPExplTypeArg _ = False
 isPFieldPun PFieldPun{} = True; isPFieldPun _ = False
 isFieldPun FieldPun{} = True; isFieldPun _ = False
 isPWildCard PWildCard{} = True; isPWildCard _ = False
@@ -146,13 +146,14 @@ isLexeme _ = False
 
 isWHNF :: Exp_ -> Bool
 isWHNF Con{} = True
-isWHNF Lit{} = True
+isWHNF (Lit _ x) = case x of String{} -> False; Int{} -> False; Frac{} -> False; _ -> True
 isWHNF Lambda{} = True
 isWHNF Tuple{} = True
 isWHNF List{} = True
 isWHNF (Paren _ x) = isWHNF x
-isWHNF RecConstr{} = True
 isWHNF (ExpTypeSig _ x _) = isWHNF x
+-- other (unknown) constructors may have bang patterns in them, so approximate
+isWHNF (App _ c@Con{} _) | prettyPrint c `elem` ["Just","Left","Right"] = True
 isWHNF _ = False
 
 
@@ -212,20 +213,17 @@ fromApps x = [x]
 -- Given (f a) b, consider the children to be: children f ++ [a,b]
 
 childrenApps :: Exp_ -> [Exp_]
-childrenApps (App _ x@App{} y) = childrenApps x ++ [y]
-childrenApps (App _ x y) = children x ++ [y]
+childrenApps (App s x y) = childrenApps x ++ [y]
 childrenApps x = children x
 
 
 descendApps :: (Exp_ -> Exp_) -> Exp_ -> Exp_
-descendApps f (App s x@App{} y) = App s (descendApps f x) (f y)
-descendApps f (App s x y) = App s (descend f x) (f y)
+descendApps f (App s x y) = App s (descendApps f x) (f y)
 descendApps f x = descend f x
 
 
-descendAppsM :: Monad m => (Exp_ -> m Exp_) -> Exp_ -> m Exp_
-descendAppsM f (App s x@App{} y) = liftM2 (App s) (descendAppsM f x) (f y)
-descendAppsM f (App s x y) = liftM2 (App s) (descendM f x) (f y)
+descendAppsM :: (Applicative m, Monad m) => (Exp_ -> m Exp_) -> Exp_ -> m Exp_
+descendAppsM f (App s x y) = liftM2 (App s) (descendAppsM f x) (f y)
 descendAppsM f x = descendM f x
 
 
@@ -235,7 +233,7 @@ universeApps x = x : concatMap universeApps (childrenApps x)
 transformApps :: (Exp_ -> Exp_) -> Exp_ -> Exp_
 transformApps f = f . descendApps (transformApps f)
 
-transformAppsM :: (Monad m) => (Exp_ -> m Exp_) -> Exp_ -> m Exp_
+transformAppsM :: (Applicative m, Monad m) => (Exp_ -> m Exp_) -> Exp_ -> m Exp_
 transformAppsM f x = f =<< descendAppsM (transformAppsM f) x
 
 
@@ -264,17 +262,23 @@ showSrcLoc (SrcLoc file line col) = take 1 file ++ f (drop 1 file) ++ ":" ++ sho
           f (x:xs) = x : f xs
           f [] = []
 
-toSrcLoc :: SrcInfo si => si -> SrcLoc
+toSrcLoc :: SrcSpanInfo -> SrcLoc
 toSrcLoc = getPointLoc
 
+toSrcSpan :: SrcSpanInfo -> SrcSpan
+toSrcSpan (SrcSpanInfo x _) = x
+
 nullSrcLoc :: SrcLoc
 nullSrcLoc = SrcLoc "" 0 0
 
+nullSrcSpan :: SrcSpan
+nullSrcSpan = mkSrcSpan nullSrcLoc nullSrcLoc
+
 an :: SrcSpanInfo
 an = toSrcInfo nullSrcLoc [] nullSrcLoc
 
 dropAnn :: Functor f => f s -> f ()
-dropAnn = fmap (const ())
+dropAnn = void
 
 
 ---------------------------------------------------------------------
