@@ -1,10 +1,11 @@
 
 module Hint.All(
     Hint(..), DeclHint, ModuHint,
-    staticHints, dynamicHints
+    builtinHints, hintRules
     ) where
 
 import Settings
+import Data.Monoid
 import Hint.Type
 
 import Hint.Match
@@ -21,8 +22,9 @@ import Hint.Extensions
 import Hint.Duplicate
 
 
-staticHints :: [(String,Hint)]
-staticHints =
+-- | A list of builtin hints, currently including entries such as @\"List\"@ and @\"Bracket\"@.
+builtinHints :: [(String,Hint)]
+builtinHints =
     ["List"       ! listHint
     ,"ListRec"    ! listRecHint
     ,"Monad"      ! monadHint
@@ -36,9 +38,10 @@ staticHints =
     ,"Duplicate"  * duplicateHint
     ]
     where
-        x!y = (x,DeclHint y)
-        x+y = (x,ModuHint y)
-        x*y = (x,CrossHint y)
+        x!y = (x,mempty{hintDecl=y})
+        x+y = (x,mempty{hintModule=y})
+        x*y = (x,mempty{hintModules=y})
 
-dynamicHints :: [Setting] -> Hint
-dynamicHints = DeclHint . readMatch
+-- | Transform a list of 'HintRule' into a 'Hint'.
+hintRules :: [HintRule] -> Hint
+hintRules xs = mempty{hintDecl=readMatch xs}
