@@ -6,7 +6,7 @@
 f = id --
 {-# LANGUAGE TotallyUnknown #-} \
 f = id
-{-# LANGUAGE Foo, Generics, ParallelListComp, ImplicitParams #-} \
+{-# LANGUAGE Foo, ParallelListComp, ImplicitParams #-} \
 f = [(a,c) | a <- b | c <- d] -- {-# LANGUAGE Foo, ParallelListComp #-}
 {-# LANGUAGE EmptyDataDecls #-} \
 data Foo
@@ -46,13 +46,21 @@ data Foo = Foo Int deriving Class --
 {-# LANGUAGE DeriveFunctor #-} \
 data Foo = Foo Int deriving Functor
 {-# LANGUAGE DeriveFunctor #-} \
-newtype Foo = Foo Int deriving Functor --
+newtype Foo = Foo Int deriving Functor
+{-# LANGUAGE GeneralizedNewtypeDeriving #-} \
+newtype Foo = Foo Int deriving Functor
+{-# LANGUAGE GeneralizedNewtypeDeriving #-} \
+newtype Foo = Foo Int deriving Data --
 {-# LANGUAGE DeriveFunctor, GeneralizedNewtypeDeriving, StandaloneDeriving #-} \
 deriving instance Functor Bar
 {-# LANGUAGE DeriveFunctor, GeneralizedNewtypeDeriving, StandaloneDeriving #-} \
 deriving instance Show Bar -- {-# LANGUAGE StandaloneDeriving #-}
 {-# LANGUAGE DeriveGeneric, GeneralizedNewtypeDeriving #-} \
 newtype Micro = Micro Int deriving Generic -- {-# LANGUAGE DeriveGeneric #-}
+{-# LANGUAGE UnboxedTuples #-} \
+f :: Int -> (# Int, Int #)
+{-# LANGUAGE UnboxedTuples #-} \
+f :: x -> (x, x); f x = (x, x) --
 </TEST>
 -}
 
@@ -66,8 +74,8 @@ import Util
 
 
 extensionsHint :: ModuHint
-extensionsHint _ x = [rawIdea Error "Unused LANGUAGE pragma" (toSrcLoc sl)
-          (prettyPrint o) (if null new then "" else prettyPrint $ LanguagePragma sl $ map (toNamed . prettyExtension) new)
+extensionsHint _ x = [rawIdea Error "Unused LANGUAGE pragma" (toSrcSpan sl)
+          (prettyPrint o) (Just $ if null new then "" else prettyPrint $ LanguagePragma sl $ map (toNamed . prettyExtension) new)
           (warnings old new)
     | not $ used TemplateHaskell x -- if TH is on, can use all other extensions programmatically
     , o@(LanguagePragma sl exts) <- modulePragmas x
@@ -87,8 +95,13 @@ warnings old new | wildcards `elem` old && wildcards `notElem` new = [Note "you
 warnings _ _ = []
 
 
+-- | Classes that don't work with newtype deriving
+noNewtypeDeriving :: [String]
+noNewtypeDeriving = ["Read","Show","Data","Typeable","Generic","Generic1"]
+
+
 usedExt :: Extension -> Module_ -> Bool
-usedExt (UnknownExtension "DeriveGeneric") = hasDerive True ["Generic","Generic1"]
+usedExt (UnknownExtension "DeriveGeneric") = hasDerive ["Generic","Generic1"]
 usedExt (EnableExtension x) = used x
 usedExt _ = const True
 
@@ -109,7 +122,6 @@ used TemplateHaskell = hasT2 (un :: (Bracket S, Splice S)) & hasS f & hasS isSpl
           f TypQuote{} = True
           f _ = False
 used ForeignFunctionInterface = hasT (un :: CallConv S)
-used Generics = hasS isPExplTypeArg
 used PatternGuards = hasS f1 & hasS f2
     where f1 (GuardedRhs _ xs _) = g xs
           f2 (GuardedAlt _ xs _) = g xs
@@ -120,17 +132,15 @@ used StandaloneDeriving = hasS isDerivDecl
 used PatternSignatures = hasS isPatTypeSig
 used RecordWildCards = hasS isPFieldWildcard & hasS isFieldWildcard
 used RecordPuns = hasS isPFieldPun & hasS isFieldPun
-used UnboxedTuples = has isBoxed
+used UnboxedTuples = has (not . isBoxed)
 used PackageImports = hasS (isJust . importPkg)
 used QuasiQuotes = hasS isQuasiQuote
 used ViewPatterns = hasS isPViewPat
-used DeriveDataTypeable = hasDerive True ["Data","Typeable"]
-used DeriveFunctor = hasDerive False ["Functor"]
-used DeriveFoldable = hasDerive False ["Foldable"]
-used DeriveTraversable = hasDerive False ["Traversable"]
-used GeneralizedNewtypeDeriving = not . null . filter (`notElem` special) . fst . derives
-    where special = ["Read","Show","Data","Typeable","Generic","Generic1"] -- these classes cannot use generalised deriving
-    -- FIXME: This special list is duplicated here, and as booleans to hasDerive
+used DeriveDataTypeable = hasDerive ["Data","Typeable"]
+used DeriveFunctor = hasDerive ["Functor"]
+used DeriveFoldable = hasDerive ["Foldable"]
+used DeriveTraversable = hasDerive ["Traversable"]
+used GeneralizedNewtypeDeriving = any (`notElem` noNewtypeDeriving) . fst . derives
 used Arrows = hasS f
     where f Proc{} = True
           f LeftArrApp{} = True
@@ -146,14 +156,13 @@ used TransformListComp = hasS f
 used x = usedExt $ UnknownExtension $ show x
 
 
-
-hasDerive :: Bool -> [String] -> Module_ -> Bool
-hasDerive nt want m = not $ null $ intersect want $ if nt then new ++ dat else dat
+hasDerive :: [String] -> Module_ -> Bool
+hasDerive want m = any (`elem` want) $ new ++ dat
     where (new,dat) = derives m
 
 
 -- | What is derived on newtype, and on data type
---   'deriving' declarations may be on either, so we approximate
+--   'deriving' declarations may be on either, so we approximate as both newtype and data
 derives :: Module_ -> ([String],[String])
 derives = concatUnzip . map f . childrenBi
     where
