@@ -9,7 +9,7 @@ yes = ['h','e','l','l','o'] -- "hello"
 
 -- [a]++b -> a : b, but only if not in a chain of ++'s
 yes = [x] ++ xs -- x : xs
-yes = "x" ++ xs -- 'x' : xs
+no = "x" ++ xs
 no = [x] ++ xs ++ ys
 no = xs ++ [x] ++ ys
 yes = [if a then b else c] ++ xs -- (if a then b else c) : xs
@@ -25,6 +25,7 @@ foo = [a b] ++ xs -- a b : xs
 
 module Hint.List where
 
+import Control.Applicative
 import Hint.Type
 
 
@@ -60,13 +61,12 @@ useString b _ = Nothing
 useList b = fmap (List an) . f True
     where
         f first x | x ~= "[]" = if first then Nothing else Just []
-        f first (view -> App2 c a b) | c ~= ":" = fmap (a:) $ f False b
+        f first (view -> App2 c a b) | c ~= ":" = (a:) <$> f False b
         f first _ = Nothing
 
 useCons False (view -> App2 op x y) | op ~= "++", Just x2 <- f x, not $ isAppend y =
         Just $ InfixApp an x2 (QConOp an $ list_cons_name an) y
     where
-        f (Lit _ (String _ [x] _)) = Just $ Lit an $ Char an x (show x)
         f (List _ [x]) = Just $ if isApp x then x else paren x
         f _ = Nothing
 useCons _ _ = Nothing
