@@ -46,7 +46,7 @@ listRecHint _ _ = concatMap f . universe
             (x, addCase) <- findCase x
             (use,severity,x) <- matchListRec x
             let y = addCase x
-            guard $ recursiveStr `notElem` vars y
+            guard $ recursiveStr `notElem` varss y
             return $ idea severity ("Use " ++ use) o y
 
 
@@ -158,7 +158,7 @@ findBranch x = do
 findPat :: [Pat_] -> Maybe ([String], Int, BList)
 findPat ps = do
     ps <- mapM readPat ps
-    [i] <- return $ findIndices isRight ps
+    [i] <- return $ findIndices isRight_ ps
     let (left,[right]) = partitionEithers ps
     return (left, i, right)
 
