@@ -29,6 +29,12 @@ concat . map f ==> concatMap f
 -- we use the associativity of (.) to add
 concat . map f . x ==> concatMap f . x
 -- currently 36 of 169 rules have (.) equivalents
+
+We see through (.) if the RHS is dull using id, e.g.
+
+not (not x) ==> x
+not . not ==> id
+not . not . x ==> x
 -}
 
 module Hint.Match(readMatch) where
@@ -40,6 +46,7 @@ import Unsafe.Coerce
 import Settings
 import Hint.Type
 import Control.Monad
+import Control.Applicative
 import Control.Arrow
 import Util
 import qualified Data.Set as Set
@@ -51,55 +58,62 @@ fmapAn = fmap (const an)
 ---------------------------------------------------------------------
 -- READ THE RULE
 
-readMatch :: [Setting] -> DeclHint
+readMatch :: [HintRule] -> DeclHint
 readMatch settings = findIdeas (concatMap readRule settings)
 
 
-readRule :: Setting -> [Setting]
-readRule m@MatchExp{lhs=(fmapAn -> lhs), rhs=(fmapAn -> rhs), side=(fmap fmapAn -> side)} =
-    (:) m{lhs=lhs,side=side,rhs=rhs} $ fromMaybe [] $ do
-        (l,v1) <- dotVersion lhs
-        (r,v2) <- dotVersion rhs
-        guard $ v1 == v2 && l /= [] && r /= [] && Set.notMember v1 (freeVars $ maybeToList side ++ l ++ r)
-        return [m{lhs=dotApps l, rhs=dotApps r, side=side}
-               ,m{lhs=dotApps (l++[toNamed v1]), rhs=dotApps (r++[toNamed v1]), side=side}]
-readRule _ = []
+readRule :: HintRule -> [HintRule]
+readRule (m@HintRule{hintRuleLHS=(fmapAn -> hintRuleLHS), hintRuleRHS=(fmapAn -> hintRuleRHS), hintRuleSide=(fmap fmapAn -> hintRuleSide)}) =
+    (:) m{hintRuleLHS=hintRuleLHS,hintRuleSide=hintRuleSide,hintRuleRHS=hintRuleRHS} $ fromMaybe [] $ do
+        (l,v1) <- dotVersion hintRuleLHS
+        (r,v2) <- dotVersion hintRuleRHS
+        guard $ v1 == v2 && l /= [] && (length l > 1 || length r > 1) && Set.notMember v1 (freeVars $ maybeToList hintRuleSide ++ l ++ r)
+        if r /= [] then return
+            [m{hintRuleLHS=dotApps l, hintRuleRHS=dotApps r, hintRuleSide=hintRuleSide}
+            ,m{hintRuleLHS=dotApps (l++[toNamed v1]), hintRuleRHS=dotApps (r++[toNamed v1]), hintRuleSide=hintRuleSide}]
+         else if length l > 1 then return
+            [m{hintRuleLHS=dotApps l, hintRuleRHS=toNamed "id", hintRuleSide=hintRuleSide}
+            ,m{hintRuleLHS=dotApps (l++[toNamed v1]), hintRuleRHS=toNamed v1, hintRuleSide=hintRuleSide}]
+         else
+            Nothing
 
 
 -- find a dot version of this rule, return the sequence of app prefixes, and the var
 dotVersion :: Exp_ -> Maybe ([Exp_], String)
 dotVersion (view -> Var_ v) | isUnifyVar v = Just ([], v)
-dotVersion (fromApps -> xs) | length xs > 1 = fmap (first (apps (init xs) :)) $ dotVersion (fromParen $ last xs)
+dotVersion (fromApps -> xs) | length xs > 1 = first (apps (init xs) :) <$> dotVersion (fromParen $ last xs)
 dotVersion _ = Nothing
 
 
 ---------------------------------------------------------------------
 -- PERFORM THE MATCHING
 
-findIdeas :: [Setting] -> Scope -> Module S -> Decl_ -> [Idea]
+findIdeas :: [HintRule] -> Scope -> Module S -> Decl_ -> [Idea]
 findIdeas matches s _ decl =
-  [ (idea (severityS m) (hintS m) x y){note=notes}
+  [ (idea (hintRuleSeverity m) (hintRuleName m) x y){ideaNote=notes}
   | decl <- case decl of InstDecl{} -> children decl; _ -> [decl]
   , (parent,x) <- universeParentExp decl, not $ isParen x, let x2 = fmapAn x
   , m <- matches, Just (y,notes) <- [matchIdea s decl m parent x2]]
 
 
-matchIdea :: Scope -> Decl_ -> Setting -> Maybe (Int, Exp_) -> Exp_ -> Maybe (Exp_,[Note])
-matchIdea s decl MatchExp{..} parent x = do
-    let nm = nameMatch scope s
-    u <- unifyExp nm True lhs x
+matchIdea :: Scope -> Decl_ -> HintRule -> Maybe (Int, Exp_) -> Exp_ -> Maybe (Exp_,[Note])
+matchIdea s decl HintRule{..} parent x = do
+    let nm a b = scopeMatch (hintRuleScope,a) (s,b)
+    u <- unifyExp nm True hintRuleLHS x
     u <- check u
-    let e = subst u rhs
-    let res = addBracket parent $ unqualify scope s u $ performEval e
-    guard $ (freeVars e Set.\\ freeVars rhs) `Set.isSubsetOf` freeVars x -- check no unexpected new free variables
-    guard $ checkSide side $ ("original",x) : ("result",res) : u
+    let e = subst u hintRuleRHS
+    let res = addBracket parent $ unqualify hintRuleScope s u $ performEval e
+    guard $ (freeVars e Set.\\ freeVars hintRuleRHS) `Set.isSubsetOf` freeVars x -- check no unexpected new free variables
+    guard $ checkSide hintRuleSide $ ("original",x) : ("result",res) : u
     guard $ checkDefine decl parent res
-    return (res,notes)
+    return (res,hintRuleNotes)
 
 
 ---------------------------------------------------------------------
 -- UNIFICATION
 
+type NameMatch = QName S -> QName S -> Bool
+
 -- unify a b = c, a[c] = b
 unify :: Data a => NameMatch -> Bool -> a -> a -> Maybe [(String,Exp_)]
 unify nm root x y | Just x <- cast x = unifyExp nm root x (unsafeCoerce y)
@@ -117,9 +131,6 @@ unifyDef nm x y = fmap concat . sequence =<< gzip (unify nm False) x y
 unifyExp :: NameMatch -> Bool -> Exp_ -> Exp_ -> Maybe [(String,Exp_)]
 unifyExp nm root x y | isParen x || isParen y = unifyExp nm root (fromParen x) (fromParen y)
 unifyExp nm root (Var _ (fromNamed -> v)) y | isUnifyVar v = Just [(v,y)]
-unifyExp nm root (Var _ (Qual _ (ModuleName _ [m]) x)) (Var _ y)
-    | Qual _ (ModuleName _ m2) y <- y, y == x = Just [([m], Var an $ UnQual an $ Ident an m2)]
-    | UnQual _ y <- y, y == x = Just [([m], Var an $ UnQual an $ Ident an "")]
 unifyExp nm root (Var _ x) (Var _ y) | nm x y = Just []
 unifyExp nm root x@(App _ x1 x2) (App _ y1 y2) =
     liftM2 (++) (unifyExp nm False x1 y1) (unifyExp nm False x2 y2) `mplus`
@@ -198,7 +209,7 @@ checkSide x bind = maybe True f x
 
         asInt :: Exp_ -> Maybe Integer
         asInt (Paren _ x) = asInt x
-        asInt (NegApp _ x) = fmap negate $ asInt x
+        asInt (NegApp _ x) = negate <$> asInt x
         asInt (Lit _ (Int _ x _)) = Just x
         asInt _ = Nothing
 
@@ -232,9 +243,9 @@ performEval x = x
 unqualify :: Scope -> Scope -> [(String,Exp_)] -> Exp_ -> Exp_
 unqualify from to subs = transformBi f
     where
-        f (Qual _ (ModuleName _ [m]) x) | Just y <- fmap fromNamed $ lookup [m] subs
+        f (Qual _ (ModuleName _ [m]) x) | Just y <- fromNamed <$> lookup [m] subs
             = if null y then UnQual an x else Qual an (ModuleName an y) x
-        f x = nameQualify from to x
+        f x = scopeMove (from,x) to
 
 
 addBracket :: Maybe (Int,Exp_) -> Exp_ -> Exp_
