@@ -1,26 +1,29 @@
-{-# LANGUAGE PatternGuards, ViewPatterns #-}
+{-# LANGUAGE PatternGuards, ViewPatterns, RecordWildCards #-}
 
 module Settings(
-    Severity(..), Note(..), showNotes, FuncName, Setting(..), isClassify, isMatchExp,
+    Severity(..), Classify(..), HintRule(..), Note(..), showNotes, Setting(..),
     defaultHintName, isUnifyVar,
-    readSettings, readPragma, findSettings
+    findSettings, readSettings,
+    readSettings2, readPragma, findSettings2
     ) where
 
 import HSE.All
 import Data.Char
 import Data.List
+import Data.Monoid
 import System.FilePath
 import Util
 
 
+defaultHintName :: String
 defaultHintName = "Use alternative"
 
 
--- | How severe an error is.
+-- | How severe an issue is.
 data Severity
-    = Ignore -- ^ Ignored errors are only returned when @--show@ is passed.
+    = Ignore -- ^ The issue has been explicitly ignored and will usually be hidden (pass @--show@ on the command line to see ignored ideas).
     | Warning -- ^ Warnings are things that some people may consider improvements, but some may not.
-    | Error -- ^ Errors are suggestions that should nearly always be a good idea to apply.
+    | Error -- ^ Errors are suggestions that are nearly always a good idea to apply.
       deriving (Eq,Ord,Show,Read,Bounded,Enum)
 
 getSeverity :: String -> Maybe Severity
@@ -32,29 +35,26 @@ getSeverity "hint"  = Just Error
 getSeverity _ = Nothing
 
 
--- (modulename,functionname)
--- either being blank implies universal matching
-type FuncName = (String,String)
-
-
 -- Any 1-letter variable names are assumed to be unification variables
 isUnifyVar :: String -> Bool
 isUnifyVar [x] = x == '?' || isAlpha x
 isUnifyVar _ = False
 
 
-addInfix x = x{infixes = infix_ (-1) ["==>"] ++ infixes x}
+addInfix = parseFlagsAddFixities $ infix_ (-1) ["==>"]
 
 
 ---------------------------------------------------------------------
 -- TYPE
 
+-- | A note describing the impact of the replacement.
 data Note
-    = IncreasesLaziness
-    | DecreasesLaziness
-    | RemovesError String -- RemovesError "on []", RemovesError "when x is negative"
-    | ValidInstance String String -- ValidInstance "Eq" "x"
-    | Note String
+    = IncreasesLaziness -- ^ The replacement is increases laziness, for example replacing @reverse (reverse x)@ with @x@ makes the code lazier.
+    | DecreasesLaziness -- ^ The replacement is decreases laziness, for example replacing @(fst x, snd x)@ with @x@ makes the code stricter.
+    | RemovesError String -- ^ The replacement removes errors, for example replacing @foldr1 (+)@ with @sum@ removes an error on @[]@, and might contain the text @\"on []\"@.
+    | ValidInstance String String -- ^ The replacement assumes standard type class lemmas, a hint with the note @ValidInstance \"Eq\" \"x\"@ might only be valid if
+                                  --   the @x@ variable has a reflexive @Eq@ instance.
+    | Note String -- ^ An arbitrary note.
       deriving (Eq,Ord)
 
 instance Show Note where
@@ -70,55 +70,99 @@ showNotes = intercalate ", " . map show . filter use
     where use ValidInstance{} = False -- Not important enough to tell an end user
           use _ = True
 
+-- | How to classify an 'Idea'. If any matching field is @\"\"@ then it matches everything.
+data Classify = Classify
+    {classifySeverity :: Severity -- ^ Severity to set the 'Idea' to.
+    ,classifyHint :: String -- ^ 'ideaHint'.
+    ,classifyModule :: String -- ^ 'ideaModule'.
+    ,classifyDecl :: String -- ^ 'ideaDecl'.
+    }
+    deriving Show
+
+-- | A @LHS ==> RHS@ style hint rule.
+data HintRule = HintRule
+    {hintRuleSeverity :: Severity -- ^ Default severity for the hint.
+    ,hintRuleName :: String -- ^ Name for the hint.
+    ,hintRuleScope :: Scope -- ^ Module scope in which the hint operates.
+    ,hintRuleLHS :: Exp SrcSpanInfo -- ^ LHS
+    ,hintRuleRHS :: Exp SrcSpanInfo -- ^ RHS
+    ,hintRuleSide :: Maybe (Exp SrcSpanInfo) -- ^ Side condition, typically specified with @where _ = ...@.
+    ,hintRuleNotes :: [Note] -- ^ Notes about application of the hint.
+    }
+    deriving Show
 
 data Setting
-    = Classify {severityS :: Severity, hintS :: String, funcS :: FuncName}
-    | MatchExp {severityS :: Severity, hintS :: String, scope :: Scope, lhs :: Exp_, rhs :: Exp_, side :: Maybe Exp_, notes :: [Note]}
+    = SettingClassify Classify
+    | SettingMatchExp HintRule
     | Builtin String -- use a builtin hint set
     | Infix Fixity
       deriving Show
 
-isClassify Classify{} = True; isClassify _ = False
-isMatchExp MatchExp{} = True; isMatchExp _ = False
-
 
 ---------------------------------------------------------------------
 -- READ A SETTINGS FILE
 
 -- Given a list of hint files to start from
 -- Return the list of settings commands
-readSettings :: FilePath -> [FilePath] -> [String] -> IO [Setting]
-readSettings dataDir files hints = do
+readSettings2 :: FilePath -> [FilePath] -> [String] -> IO [Setting]
+readSettings2 dataDir files hints = do
     (builtin,mods) <- fmap unzipEither $ concatMapM (readHints dataDir) $ map Right files ++ map Left hints
-    let f m = concatMap (readSetting $ moduleScope m) $ concatMap getEquations $ moduleDecls m
-    return $ map Builtin builtin ++ concatMap f mods
+    return $ map Builtin builtin ++ concatMap moduleSettings_ mods
+
+moduleSettings_ :: Module SrcSpanInfo -> [Setting]
+moduleSettings_ m = concatMap (readSetting $ scopeCreate m) $ concatMap getEquations $
+                       [AnnPragma l x | AnnModulePragma l x <- modulePragmas m] ++ moduleDecls m
+
+-- | Given a module containing HLint settings information return the 'Classify' rules and the 'HintRule' expressions.
+--   Any fixity declarations will be discarded, but any other unrecognised elements will result in an exception.
+readSettings :: Module SrcSpanInfo -> ([Classify], [HintRule])
+readSettings m = ([x | SettingClassify x <- xs], [x | SettingMatchExp x <- xs])
+    where xs = moduleSettings_ m
 
 
--- Read a hint file, and all hint files it imports
 readHints :: FilePath -> Either String FilePath -> IO [Either String Module_]
-readHints dataDir file = do
-    let flags = addInfix parseFlags
-    y <- parseResult $ either (parseString flags "CommandLine") (parseFile flags) file
-    ys <- concatM [f $ fromNamed $ importModule i | i <- moduleImports y, importPkg i `elem` [Just "hint", Just "hlint"]]
-    return $ Right y:ys
+readHints datadir x = do
+    (builtin,ms) <- case x of
+        Left src -> findSettings datadir "CommandLine" (Just src)
+        Right file -> findSettings datadir file Nothing
+    return $ map Left builtin ++ map Right ms
+
+
+-- | Given the data directory (where the @hlint@ data files reside, see 'getHLintDataDir'),
+--   and a filename to read, and optionally that file's contents, produce a pair containing:
+--
+-- 1. Builtin hints to use, e.g. @"List"@, which should be resolved using 'builtinHints'.
+--
+-- 1. A list of modules containing hints, suitable for processing with 'readSettings'.
+--
+--   Any parse failures will result in an exception.
+findSettings :: FilePath -> FilePath -> Maybe String -> IO ([String], [Module SrcSpanInfo])
+findSettings dataDir file contents = do
+    let flags = addInfix defaultParseFlags
+    res <- parseModuleEx flags file contents
+    case res of
+        Left (ParseError sl msg err) -> exitMessage $ "Parse failure at " ++ showSrcLoc sl ++ ": " ++ msg ++ "\n" ++ err
+        Right m -> do
+            ys <- sequence [f $ fromNamed $ importModule i | i <- moduleImports m, importPkg i `elem` [Just "hint", Just "hlint"]]
+            return $ concat2 $ ([],[m]) : ys
     where
-        f x | "HLint.Builtin." `isPrefixOf` x = return [Left $ drop 14 x]
-            | "HLint." `isPrefixOf` x = readHints dataDir $ Right $ dataDir </> drop 6 x <.> "hs"
-            | otherwise = readHints dataDir $ Right $ x <.> "hs"
+        f x | Just x <- "HLint.Builtin." `stripPrefix` x = return ([x],[])
+            | Just x <- "HLint." `stripPrefix` x = findSettings dataDir (dataDir </> x <.> "hs") Nothing
+            | otherwise = findSettings dataDir (x <.> "hs") Nothing
 
 
 readSetting :: Scope -> Decl_ -> [Setting]
 readSetting s (FunBind _ [Match _ (Ident _ (getSeverity -> Just severity)) pats (UnGuardedRhs _ bod) bind])
     | InfixApp _ lhs op rhs <- bod, opExp op ~= "==>" =
         let (a,b) = readSide $ childrenBi bind in
-        [MatchExp severity (headDef defaultHintName names) s (fromParen lhs) (fromParen rhs) a b]
-    | otherwise = [Classify severity n func | n <- names2, func <- readFuncs bod]
+        [SettingMatchExp $ HintRule severity (headDef defaultHintName names) s (fromParen lhs) (fromParen rhs) a b]
+    | otherwise = [SettingClassify $ Classify severity n a b | n <- names2, (a,b) <- readFuncs bod]
     where
         names = filter notNull $ getNames pats bod
         names2 = ["" | null names] ++ names
 
 readSetting s x | "test" `isPrefixOf` map toLower (fromNamed x) = []
-readSetting s x@AnnPragma{} | Just y <- readPragma x = [y]
+readSetting s (AnnPragma _ x) | Just y <- readPragma x = [SettingClassify y]
 readSetting s (PatBind an (PVar _ name) _ bod bind) = readSetting s $ FunBind an [Match an name [] bod bind]
 readSetting s (FunBind an xs) | length xs /= 1 = concatMap (readSetting s . FunBind an . return) xs
 readSetting s (SpliceDecl an (App _ (Var _ x) (Lit _ y))) = readSetting s $ FunBind an [Match an (toNamed $ fromNamed x) [PLit an y] (UnGuardedRhs an $ Lit an $ String an "" "") Nothing]
@@ -127,22 +171,20 @@ readSetting s x = errorOn x "bad hint"
 
 
 -- return Nothing if it is not an HLint pragma, otherwise all the settings
-readPragma :: Decl_ -> Maybe Setting
-readPragma o@(AnnPragma _ p) = f p
+readPragma :: Annotation S -> Maybe Classify
+readPragma o = case o of
+    Ann _ name x -> f (fromNamed name) x
+    TypeAnn _ name x -> f (fromNamed name) x
+    ModuleAnn _ x -> f "" x
     where
-        f (Ann _ name x) = g (fromNamed name) x
-        f (TypeAnn _ name x) = g (fromNamed name) x
-        f (ModuleAnn _ x) = g "" x
-
-        g name (Lit _ (String _ s _)) | "hlint:" `isPrefixOf` map toLower s =
+        f name (Lit _ (String _ s _)) | "hlint:" `isPrefixOf` map toLower s =
                 case getSeverity a of
                     Nothing -> errorOn o "bad classify pragma"
-                    Just severity -> Just $ Classify severity (ltrim b) ("",name)
+                    Just severity -> Just $ Classify severity (ltrim b) "" name
             where (a,b) = break isSpace $ ltrim $ drop 6 s
-        g name (Paren _ x) = g name x
-        g name (ExpTypeSig _ x _) = g name x
-        g _ _ = Nothing
-readPragma _ = Nothing
+        f name (Paren _ x) = f name x
+        f name (ExpTypeSig _ x _) = f name x
+        f _ _ = Nothing
 
 
 readSide :: [Decl_] -> (Maybe Exp_, [Note])
@@ -168,7 +210,7 @@ readSide = foldl f (Nothing,[])
 
 
 -- Note: Foo may be ("","Foo") or ("Foo",""), return both
-readFuncs :: Exp_ -> [FuncName]
+readFuncs :: Exp_ -> [(String, String)]
 readFuncs (App _ x y) = readFuncs x ++ readFuncs y
 readFuncs (Lit _ (String _ "" _)) = [("","")]
 readFuncs (Var _ (UnQual _ name)) = [("",fromNamed name)]
@@ -201,16 +243,16 @@ errorOn val msg = exitMessage $
 -- FIND SETTINGS IN A SOURCE FILE
 
 -- find definitions in a source file
-findSettings :: ParseFlags -> FilePath -> IO (String, [Setting])
-findSettings flags file = do
-    x <- parseFile flags file
-    case snd x of
-        ParseFailed sl msg ->
+findSettings2 :: ParseFlags -> FilePath -> IO (String, [Setting])
+findSettings2 flags file = do
+    x <- parseModuleEx flags file Nothing
+    case x of
+        Left (ParseError sl msg _) ->
             return ("-- Parse error " ++ showSrcLoc sl ++ ": " ++ msg, [])
-        ParseOk m -> do
+        Right m -> do
             let xs = concatMap (findSetting $ UnQual an) (moduleDecls m)
                 s = unlines $ ["-- hints found in " ++ file] ++ map prettyPrint xs ++ ["-- no hints found" | null xs]
-                r = concatMap (readSetting emptyScope) xs
+                r = concatMap (readSetting mempty) xs
             return (s,r)
 
 
