@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, ExistentialQuantification, Rank2Types, PatternGuards #-}
+{-# LANGUAGE ExistentialQuantification, Rank2Types, PatternGuards #-}
 
 module Util where
 
@@ -19,10 +19,7 @@ import Unsafe.Coerce
 import Data.Data
 import Data.Generics.Uniplate.Operations
 import Language.Haskell.Exts.Extension
-
-#if __GLASGOW_HASKELL__ >= 612
 import GHC.IO.Handle(hDuplicate,hDuplicateTo)
-#endif
 
 
 ---------------------------------------------------------------------
@@ -32,8 +29,8 @@ getDirectoryContentsRecursive :: FilePath -> IO [FilePath]
 getDirectoryContentsRecursive dir = do
     xs <- getDirectoryContents dir
     (dirs,files) <- partitionM doesDirectoryExist [dir </> x | x <- xs, not $ isBadDir x]
-    rest <- concatMapM getDirectoryContentsRecursive dirs
-    return $ files++rest
+    rest <- concatMapM getDirectoryContentsRecursive $ sort dirs
+    return $ sort files ++ rest
     where
         isBadDir x = "." `isPrefixOf` x || "_" `isPrefixOf` x
 
@@ -93,6 +90,9 @@ limit n s = if null post then s else pre ++ "..."
 ltrim :: String -> String
 ltrim = dropWhile isSpace
 
+rtrim :: String -> String
+rtrim = reverse . ltrim . reverse
+
 trimBy :: (a -> Bool) -> [a] -> [a]
 trimBy f = reverse . dropWhile f . reverse . dropWhile f
 
@@ -123,32 +123,54 @@ concatUnzip = (concat *** concat) . unzip
 swap :: (a,b) -> (b,a)
 swap (a,b) = (b,a)
 
+fst3 :: (a,b,c) -> a
+fst3 (a,b,c) = a
+
+snd3 :: (a,b,c) -> b
+snd3 (a,b,c) = b
+
+thd3 :: (a,b,c) -> c
+thd3 (a,b,c) = c
+
+concat3 :: [([a],[b],[c])] -> ([a],[b],[c])
+concat3 xs = (concat a, concat b, concat c)
+    where (a,b,c) = unzip3 xs
+
+concat2 :: [([a],[b])] -> ([a],[b])
+concat2 xs = (concat a, concat b)
+    where (a,b) = unzip xs
+
 
 ---------------------------------------------------------------------
 -- SYSTEM.IO
 
--- | An encoding is a function to change a handle to a particular encoding
+-- | An 'Encoding' represents how characters are stored in a file. Created with
+--   'defaultEncoding' or 'readEncoding' and used with 'useEncoding'.
 data Encoding = Encoding_Internal (Maybe (Handle -> IO ()))
 
+-- | The system default encoding.
 defaultEncoding :: Encoding
 defaultEncoding = Encoding_Internal Nothing
 
+-- | Apply an encoding to a 'Handle'.
+useEncoding :: Handle -> Encoding -> IO ()
+useEncoding h (Encoding_Internal x) = maybe (return ()) ($ h) x
 
 readFileEncoding :: Encoding -> FilePath -> IO String
-readFileEncoding (Encoding_Internal x) file = case x of
-    Nothing -> if file == "-" then getContents else readFile file
-    Just set -> do
-        h <- if file == "-" then return stdin else openFile file ReadMode
-        set h
-        hGetContents h
+readFileEncoding enc file = do
+    h <- if file == "-" then return stdin else openFile file ReadMode
+    useEncoding h enc
+    hGetContents h
 
 
+-- | Create an encoding from a string, or throw an error if the encoding is not known.
+--   Accepts many encodings including @locale@, @utf-8@ and all those supported by the
+--   GHC @mkTextEncoding@ function.
+readEncoding :: String -> IO Encoding
 -- GHC's mkTextEncoding function is fairly poor - it doesn't support lots of fun things,
 -- so we fake them up, and then try mkTextEncoding last
-newEncoding :: String -> IO Encoding
-newEncoding "" = return defaultEncoding
-#if __GLASGOW_HASKELL__ >= 612
-newEncoding enc
+readEncoding "" = return defaultEncoding
+readEncoding enc
         | Just e <- lookup (f enc) [(f a, b) | (as,b) <- encs, a <- as] = return $ wrap e
         | otherwise = do
             res <- try $ mkTextEncoding enc :: IO (Either SomeException TextEncoding)
@@ -178,11 +200,6 @@ newEncoding enc
                   ,"UTF-32" * utf16
                   ,"UTF-32LE" * utf16le
                   ,"UTF-32BE" * utf16be]
-#else
-newEncoding enc = do
-    putStrLn "Warning: Text encodings are not supported with HLint compiled by GHC 6.10"
-    return defaultEncoding
-#endif
 
 
 exitMessage :: String -> a
@@ -192,10 +209,7 @@ exitMessage msg = unsafePerformIO $ do
 
 
 -- FIXME: This could use a lot more bracket calls!
-captureOutput :: IO () -> IO (Maybe String)
-#if __GLASGOW_HASKELL__ < 612
-captureOutput act = return Nothing
-#else
+captureOutput :: IO () -> IO String
 captureOutput act = do
     tmp <- getTemporaryDirectory
     (f,h) <- openTempFile tmp "hlint"
@@ -209,8 +223,7 @@ captureOutput act = do
     hDuplicateTo ste stderr
     res <- readFile' f
     removeFile f
-    return $ Just res
-#endif
+    return res
 
 
 -- FIXME: Should use strict ByteString
@@ -258,4 +271,6 @@ badExtensions =
     [Arrows -- steals proc
     ,TransformListComp -- steals the group keyword
     ,XmlSyntax, RegularPatterns -- steals a-b
+    ,UnboxedTuples -- breaks (#) lens operator
+    ,QuasiQuotes -- breaks [x| ...], making whitespace free list comps break
     ]
